from loguru import logger
import subprocess
import shlex

from ..exceptions import BinaryNotFound
from ._base import BaseNotifier

try:
    from jeepney import DBusAddress, new_method_call
    from jeepney.io.blocking import open_dbus_connection
    from shutil import which

    NOTIFY = which('notify-send') # alternatively: from ctypes.util import find_library 

    if NOTIFY:
        logger.info("libnotify found, using it for notifications")
    else: # check if dbus is available
        import os
        _dbus_address = os.getenv("DBUS_SESSION_BUS_ADDRESS")
        if _dbus_address:
            logger.info("Jeepney and Dbus is available. Using DBUS for notifications..")
        else:
            raise ImportError

    APLAY = which('aplay')

    if APLAY == None:
        logger.debug("aplay binary not installed.. audio will not work!")


except ImportError:
    logger.error("libnotify nor DBUS installed.")


class LinuxNotifierLibNotify(BaseNotifier):
    def __init__(self, **kwargs):
        """Main Linux Notification Class

        This uses libnotify's tool of notfiy-send.
        """
        pass

    def send_notification(
        self,
        notification_title,
        notification_subtitle,
        notification_icon,
        notification_audio,
        **kwargs,
    ):
        try:
            notification_title = " " if notification_title == "" else notification_title
            notification_subtitle = (
                " " if notification_subtitle == "" else notification_subtitle
            )

            generated_command = [
                NOTIFY,
                notification_title,
                notification_subtitle,
            ]

            if notification_icon:
                generated_command.append(f"--icon={shlex.quote(notification_icon)}")

            if kwargs.get("application_name"):
                generated_command.append(
                    f"--app-name={shlex.quote(kwargs.get('application_name'))}"
                )

            if kwargs.get('notification_urgency'):
                generated_command.extend(["-u", kwargs.get('notification_urgency')])

            logger.debug(f"Generated command: {generated_command}")
            if notification_audio:

                if APLAY == None:
                    raise BinaryNotFound("aplay (Alsa)")

                subprocess.Popen(
                    [APLAY, notification_audio],
                    stdout=subprocess.DEVNULL,
                    stderr=subprocess.STDOUT,
                )

            subprocess.check_output(generated_command)
            return True
        except subprocess.CalledProcessError:
            logger.exception("Unable to send notification.")
            return False
        except Exception:
            logger.exception("Unhandled exception for sending notification.")
            return False


class LinuxNotifier(BaseNotifier):
    def __init__(self, **kwargs):
        """Main Linux Notification Class (Dbus)

        This uses jeepney library as the dbus communicator

        """

        self._dbus_notifications = DBusAddress(
            "/org/freedesktop/Notifications",
            bus_name="org.freedesktop.Notifications",
            interface="org.freedesktop.Notifications",
        )


    def send_notification(
        self,
        notification_title,
        notification_subtitle,
        notification_icon,
        notification_audio,
        **kwargs,
    ):
        try:
            _attempt_to_open_dbus_connection = open_dbus_connection(bus="SESSION")
            logger.debug("linux: opened dbus connection")
        except Exception:
            logger.exception("issue with opening DBUS connection!")
            return False

        try:
            notification_title = " " if notification_title == "" else notification_title
            notification_subtitle = (
                " " if notification_subtitle == "" else notification_subtitle
            )

            if notification_audio:
                # TODO: https://specifications.freedesktop.org/notification-spec/latest/ar01s09.html
                # use sound param instead of relying on alsa?

                if APLAY == None:
                    raise BinaryNotFound("aplay (Alsa)")

                subprocess.Popen(
                    [APLAY, notification_audio],
                    stdout=subprocess.DEVNULL,
                    stderr=subprocess.STDOUT,
                )
            create_notification = new_method_call(
                self._dbus_notifications,
                "Notify",
                "susssasa{sv}i",
                (
                    kwargs.get("application_name"),  # App name
                    0,  # Not replacing any previous notification
                    notification_icon if notification_icon else "",  # Icon
                    notification_title,  # Summary
                    notification_subtitle,
                    [],
                    {},  # Actions, hints
                    -1,  # expire_timeout (-1 = default)
                ),
            )
            reply = _attempt_to_open_dbus_connection.send_and_get_reply(
                create_notification, timeout=2
            )
            logger.debug(f"confirmed notification sent! id: {reply}")
            _attempt_to_open_dbus_connection.close()
            return True

        except Exception:
            logger.exception("issue with sending through dbus!")
            return False



# Custom and Clear Exceptions for Tellit


class BaseTellitException(Exception):
    """Base Exception to be Inheritied"""


class UnsupportedPlatform(BaseTellitException):
    """Unsupported Platform, notify-py might not work as expected"""

    def __init__(self, platform):
        self.platform = platform

    def __repr__(self):
        return f"{self.platform} is not supported."

    def __str__(self):
        return f"{self.platform} is not supported."


class InvalidAudioPath(BaseTellitException):
    """Audio path provided is invalid."""

    def __repr__(self):
        return f"Unable to find audio path. Please check if it exists."

    def __str__(self):
        return f"Unable to find audio path. Please check if it exists."


class InvalidAudioFormat(BaseTellitException):
    """The custom audio provided is not a supported file"""

    def __repr__(self):
        return f"Only .wav files are supported."

    def __str__(self):
        return f"Only .wav files are supported."


class InvalidIconPath(BaseTellitException):
    """Icon Path Provided is Invalid"""

    def __repr__(self):
        return f"Unable to find icon path. Please check if it exists."

    def __str__(self):
        return f"Unable to find icon path. Please check if it exists."


class NotificationFailure(BaseTellitException):
    """Overall function failed"""

    pass


class BinaryNotFound(BaseTellitException):
    """ " A specified binary requirement was not found"""

    def __init__(self, binary):
        self.binary = binary

    def __repr__(self):
        return f"Unable find required {self.binary}. Please check if it's installed."

    def __str__(self):
        return f"Unable to find required {self.binary}. Please check if it's installed."


class InvalidMacOSNotificator(BaseTellitException):
    """The provided notificator is invalid."""

    def __str__(self):
        return f"The provided notificator is invalid. Please read the documentation for more information."

    def __repr__(self):
        return f"The provided notificator is invalid. Please read the documentation for more information."


class LinuxDbusException(Exception):
    """This error is raised when a connection with dbus is interrupted or is unable to be established"""

    pass



from .notify import Notify
import argparse


def _enable_argument_parser() -> argparse.ArgumentParser:
    argument_parser = argparse.ArgumentParser(
        description="Send cross-platform desktop notifications."
    )

    # Required Arguments.
    argument_parser.add_argument(
        "--title",
        "-t",
        dest="userCreatedTitle",
        help="Notification Title",
        required=True,
    )
    argument_parser.add_argument(
        "--message",
        "-m",
        dest="userCreatedMessage",
        help="Notification Message",
        required=True,
    )
    argument_parser.add_argument(
        "--applicationName",
        "-a",
        dest="userCreatedApplicationName",
        help="Notification application name",
        required=True,
    )
    # Optional Arguments
    argument_parser.add_argument(
        "--iconPath",
        "-i",
        dest="userCreatedIconPath",
        help="Notification Icon. (Not available on macOS). Specify a direct path to icon.",
        required=False,
    )
    argument_parser.add_argument(
        "--soundPath",
        "-s",
        dest="userCreatedSoundPath",
        help="Notification Sound. Specify a direct path.",
        required=False,
    )

    # Extras
    argument_parser.add_argument(
        "--enableLogging", dest="userCreatedEnableLogging", action="store_true"
    )
    argument_parser.add_argument(
        "--overridePlatform",
        dest="userCreatedOverridePlatform",
        help="Overrides the check for determining appropriate notifier. (Windows, Darwin, Linux).",
        required=False,
    )

    return argument_parser


def entry():
    """Entrypoint for CLI (tellit)"""
    parser = _enable_argument_parser()
    arguments_recieved = parser.parse_args()

    _current_extra_built_kwargs = {}

    if arguments_recieved.userCreatedOverridePlatform in ["Windows", "Darwin", "Linux"]:
        _current_extra_built_kwargs[
            "use_custom_notifier"
        ] = arguments_recieved.userCreatedOverridePlatform

    if arguments_recieved.userCreatedEnableLogging == True:
        _current_extra_built_kwargs["enable_logging"] = True

    if arguments_recieved.userCreatedSoundPath:
        _current_extra_built_kwargs[
            "default_notification_audio"
        ] = arguments_recieved.userCreatedSoundPath

    if arguments_recieved.userCreatedIconPath:
        _current_extra_built_kwargs[
            "default_notification_icon"
        ] = arguments_recieved.userCreatedIconPath
    Notify(
        default_notification_title=arguments_recieved.userCreatedTitle,
        default_notification_message=arguments_recieved.userCreatedMessage,
        default_notification_application_name=arguments_recieved.userCreatedApplicationName,
        **_current_extra_built_kwargs
    ).send()



import os
import pathlib
import subprocess
import shlex

from loguru import logger
from ..exceptions import BinaryNotFound, NotificationFailure, InvalidMacOSNotificator
from ._base import BaseNotifier


class MacOSNotifier(BaseNotifier):
    def __init__(self, **kwargs):
        """Main macOS Notification System, supplied by a custom-made notificator app.
        Icon Support is **not** supported. You'll need to create your own bundle for that.
        """

        if kwargs.get("custom_mac_notificator"):
            """This optional kwarg exists for the use of using a custom (made) notificator without building a .whl"""
            selected_custom_notificator = kwargs.get("custom_mac_notificator")
            if (
                pathlib.Path(selected_custom_notificator)
                / "Contents/Resources/Scripts/notificator"
            ).exists():
                current_selected_binary = pathlib.Path(
                    selected_custom_notificator
                ).absolute()
                if os.access(
                    (
                        current_selected_binary
                        / "Contents/Resources/Scripts/notificator"
                    ),
                    os.X_OK,
                ):
                    self._notificator_binary = str(
                        current_selected_binary
                        / "Contents/Resources/Scripts/notificator"
                    )
                else:
                    raise InvalidMacOSNotificator(
                        "Unable to access binary, you might need to update the permissions."
                    )
            else:
                raise InvalidMacOSNotificator
        else:
            call_find_notificator = self._find_bundled_notificator()
            if not call_find_notificator:
                logger.error("Unable to find Bundled Notificator")
                raise BinaryNotFound("bundled notifcator.")
            if call_find_notificator:
                self._notificator_binary = call_find_notificator

        call_find_afplay = self._find_installed_afplay()
        if not call_find_afplay:
            # no Afplay is available.
            self._afplay_binary = False
        else:
            self._afplay_binary = call_find_afplay

    @staticmethod
    def _find_bundled_notificator():
        """Gets the bundled Notifcator"""
        try:
            current_bundled = os.path.join(
                os.path.dirname(__file__),
                "binaries/Notificator.app/Contents/Resources/Scripts/notificator",
            )
            if pathlib.Path(current_bundled).exists():
                return current_bundled
            raise BinaryNotFound("bundled notifier.app")
        except Exception:
            logger.exception("Unhandled exception for finding afplay.")
            raise

    @staticmethod
    def _find_installed_afplay():
        """Function to find the path for afplay"""
        try:
            run_which_for_aplay = subprocess.check_output(["which", "afplay"])
            return run_which_for_aplay.decode("utf-8")
        except subprocess.CalledProcessError:
            logger.exception("Unable to find aplay.")
            return False
        except Exception:
            logger.exception("Unhandled exception for finding aplay.")
            return False

    def send_notification(
        self,
        notification_title,
        notification_subtitle,
        application_name,
        notification_audio,
        **kwargs
    ):
        if kwargs.get("notification_icon"):
            logger.warning(
                "Notification icon is not supported. Read the docs for more information."
            )

        try:

            notification_title = " " if notification_title == "" else notification_title
            notification_subtitle = (
                " " if notification_subtitle == "" else notification_subtitle
            )

            if notification_audio:

                if self._afplay_binary == False:
                    raise BinaryNotFound("afplay")

                subprocess.Popen(
                    [self._afplay_binary.strip(), notification_audio],
                    stdout=subprocess.DEVNULL,
                    stderr=subprocess.STDOUT,
                )

            generated_command = [
                self._notificator_binary,
                "--title",
                application_name,
                "--subtitle",
                notification_title,
                "--message",
                notification_subtitle,
            ]

            subprocess.check_output(generated_command)
            return True
        except subprocess.CalledProcessError:
            logger.exception("Unable to send notification.")
            return False
        except Exception:
            logger.exception("Unhandled Exception for sending notifications")
            return False



class BaseNotifier(object):
    """This is a base object to be inheritied by each notifier. You can inherit this if you choose to create your own notifier."""

    def send_notification(self, **kwargs):
        raise NotImplementedError(
            "You'll need to expose a send_notification method in your notifier."
        )



import platform
import os
import pathlib
import threading


from loguru import logger
from .exceptions import (
    UnsupportedPlatform,
    InvalidAudioPath,
    InvalidIconPath,
    NotificationFailure,
    BinaryNotFound,
    InvalidAudioFormat,
)

from .os_notifiers._base import BaseNotifier


class Notify:
    def __init__(
        self,
        default_notification_title="Default Title",
        default_notification_message="Default Message",
        default_notification_application_name="Python Application (notify.py)",
        default_notification_urgency='normal',
        default_notification_icon=None,
        default_notification_audio=None,
        enable_logging=False,
        **kwargs,
    ):
        """Main Notify Class.

        Optional Arugments:
            override_detected_notification_system: Optional Kwarg that allows for the use of overrding the detected notifier.
            disable_logging: Optional Kwarg that will disable stdout logging from this library.
            custom_mac_notificator: Optional Kwarg for a custom mac notifier. (Probably because you want to change the icon.). This is a direct path to the parent directory (.app).

        """

        if not enable_logging:
            check_if_enable_logging_env = os.getenv("tellitEnableLogging")
            if check_if_enable_logging_env:
                logger.warning(
                    f"System environment variable for enabling logging is active. Ignoring parameter passed. ({enable_logging})"
                )
            else:
                logger.disable("tellit")
        else:
            logger.info("Logging is enabled.")

        if kwargs.get("use_custom_notifier"):
            """
            This optional kwarg allows for the use of overriding the detected notifier.
            Use at your own risk
            """
            selected_override = kwargs.get("use_custom_notifier")
            if issubclass(selected_override, BaseNotifier):
                self._notifier_detect = selected_override
            else:
                raise ValueError("Overrided Notifier must inherit from BaseNotifier.")
        else:
            check_if_user_override_detection = kwargs.get(
                "override_detected_notification_system"
            )
            if check_if_user_override_detection:
                self._notifier_detect = self._selected_notification_system(**kwargs)
            else:
                self._notifier_detect = self._selected_notification_system()

        # Initialize.
        self._notifier = self._notifier_detect(**kwargs)

        # Set the defaults.
        self._notification_title = default_notification_title
        self._notification_message = default_notification_message
        self._notification_application_name = default_notification_application_name
        self._notification_urgency = default_notification_urgency

        # These defaults require verification
        if default_notification_icon:
            self._notification_icon = self._verify_icon_path(default_notification_icon)
        else:
            self._notification_icon = str(
                os.path.join(os.path.dirname(__file__), "py-logo.png")
            )

        if default_notification_audio:
            self._notification_audio = self._verify_audio_path(
                default_notification_audio
            )
        else:
            self._notification_audio = None

    @staticmethod
    def _selected_notification_system(
        override_detection: str = False,
        override_windows_version_detection: bool = False,
        linux_use_legacy_notifier: bool = False,
    ):

        if override_detection:
            logger.info(f"chosen to override to {override_detection}.")
            selected_platform = override_detection
        else:
            selected_platform = platform.system()

        if selected_platform == "Darwin":
            from .os_notifiers.macos import MacOSNotifier

            return MacOSNotifier
        elif selected_platform == "Windows":
            if platform.release() in {"10", "11"}:
                from .os_notifiers.windows import WindowsNotifier

                return WindowsNotifier

            if override_windows_version_detection == True:
                from .os_notifiers.windows import WindowsNotifier

                return WindowsNotifier

            raise UnsupportedPlatform(
                f"This version of Windows ({platform.release()}) is not supported."
            )
        else:
            if selected_platform != "Linux":
                logger.warning(f'{selected_platform} might not be supported!')

            if linux_use_legacy_notifier:
                from .os_notifiers.linux import LinuxNotifierLibNotify

                return LinuxNotifierLibNotify
            else:

                from .os_notifiers.linux import NOTIFY

                if NOTIFY:
                    from .os_notifiers.linux import LinuxNotifierLibNotify

                    return LinuxNotifierLibNotify
                else:
                    from .os_notifiers.linux import LinuxNotifier

                    return LinuxNotifier

    @staticmethod
    def _verify_audio_path(new_audio_path):
        # we currently only support .wav files
        if not new_audio_path.endswith(".wav"):
            raise InvalidAudioFormat

        # first detect if it already exists.
        if pathlib.Path(new_audio_path).exists():
            return str(pathlib.Path(new_audio_path).absolute())
        else:
            # Ok doesn't exist, let's try a join
            if pathlib.Path(
                os.path.join(os.path.dirname(__file__), new_audio_path)
            ).exists():
                return str(os.path.join(os.path.dirname(__file__), new_audio_path))
            else:
                raise InvalidAudioPath(
                    f"Could not find specified audio path to '{new_audio_path}'. Please check if it exists."
                )

    @staticmethod
    def _verify_icon_path(new_icon_path):
        # first detect if it already exists.
        if pathlib.Path(new_icon_path).exists():
            return str(pathlib.Path(new_icon_path).absolute())
        else:
            # Ok doesn't exist, let's try a join
            if pathlib.Path(
                os.path.join(os.path.dirname(__file__), new_icon_path)
            ).exists():
                return os.path.join(os.path.dirname(__file__), new_icon_path)
            else:
                raise InvalidIconPath(
                    f"Could not find specified icon path to '{new_icon_path}'. Please check if it exists."
                )

    @property
    def audio(self):
        """A direct path to a '.wav' audio file.

        Returns:
            str: direct path to '.wav' audio file.
        """
        return self._notification_audio

    @audio.setter
    def audio(self, new_audio_path):
        self._notification_audio = self._verify_audio_path(new_audio_path)

    @property
    def icon(self):
        """A direct path to a '.png' image file.
        Note: .jpg might work, but hasn't been fully tested.
        macOS does not support setting this attribute.

        Returns:
            str: A direct path to a '.png' image file.
        """
        return self._notification_icon

    @icon.setter
    def icon(self, new_icon_path):
        self._notification_icon = self._verify_icon_path(new_icon_path)

    @property
    def title(self):
        """The top (often bolded) message for the notification.

        macOS (as of 0.2.0) the application name takes the bolded area, while the title is directly below it.

        Returns:
            str: The top (often bolded) message for the notification.
        """
        return self._notification_title

    @title.setter
    def title(self, new_title):
        self._notification_title = new_title

    @property
    def message(self):
        """The main message to be displayed in the notification. Directly below the title.

        Returns:
            str: The message for the notification.
        """
        return self._notification_message

    @message.setter
    def message(self, new_message):
        self._notification_message = new_message

    @property
    def application_name(self):
        """The application name that will be displayed (if the platform allows it.)
        Windows and macOS requires an application name to be displayed.

        Returns:
            str: the application name
        """
        return self._notification_application_name

    @application_name.setter
    def application_name(self, new_application_name):
        self._notification_application_name = new_application_name

    @property
    def urgency(self):
        """The urgency of the notification (low, normal, critical) 
        Works only with libnotify (Linux), as of now

        Returns:
            str: The urgency of the notification.
        """
        return self._notification_urgency

    @urgency.setter
    def urgency(self, new_urgency):
        self._notification_urgency = new_urgency

    def send(self, block=True):
        """Main send function. This will take all attributes sent and forward to
        send_notification.

        Args:
            block (bool, optional): Optional value to not to block the main application thread. If enabled this won't return a bool. Defaults to True.

        Returns:
            bool: as long as the block isn't set to False.
        """
        # if block is True, wait for the notification to complete and return if it was successful
        # else start the thread and return a threading.Event that will determine when the notification was successful
        event = threading.Event()
        try:
            thread = threading.Thread(
                target=lambda: self.start_notification_thread(event)
            )
            thread.name = "notify.py"
            thread.start()
            if block:
                thread.join(timeout=35)
                return event.is_set()
            return event
        except Exception:
            logger.exception("Unhandled exception for sending notification.")
            raise

    def start_notification_thread(self, event):
        """Function for sending notification via a seperate thread.
        You don't need to call this directly. Do .send(block=False)

        Args:
            event (threading.Thread): event to be recieved.
        """
        result = self.send_notification(
            supplied_title=self._notification_title,
            supplied_message=self._notification_message,
            supplied_application_name=self._notification_application_name,
            supplied_urgency=self._notification_urgency,
            supplied_icon_path=self._notification_icon,
            supplied_audio_path=self._notification_audio,
        )
        if result:
            event.set()
        else:
            event.clear()

    def send_notification(
        self,
        supplied_title,
        supplied_message,
        supplied_application_name,
        supplied_urgency,
        supplied_icon_path,
        supplied_audio_path,
    ):
        """A function to handles sending all required variables to respected OS-Notifier.

        Args:
            supplied_title str: Title for notification
            supplied_message str: Message for notification
            supplied_application_name str: Application name for notification (if platform needs it)
            supplied_urgency str: low, normal, critical | Notification urgency
            supplied_icon_path str: Direct path to custom icon
            supplied_audio_path str: Direct path to custom audio

        Raises:
            NotificationFailure: If there was an Exception in sending the notification.

        Returns:
            bool: True if the notification was sent.
        """
        try:
            attempt_to_send_notifiation = self._notifier.send_notification(
                notification_title=str(supplied_title),
                notification_subtitle=str(supplied_message),
                application_name=str(supplied_application_name),
                notification_urgency=str(supplied_urgency),
                notification_icon=str(supplied_icon_path),
                notification_audio=str(supplied_audio_path)
                if supplied_audio_path
                else None,
            )
            if attempt_to_send_notifiation:
                logger.info("Sent notification.")
            else:
                logger.info("unable to send notification.")

            return attempt_to_send_notifiation
        except Exception:
            logger.exception("Exception on sending notification.")
            raise NotificationFailure



import pathlib
import os
import subprocess
from xml.etree import ElementTree
import tempfile
import uuid
import codecs

from loguru import logger
from ._base import BaseNotifier


class WindowsNotifier(BaseNotifier):
    def __init__(self, **kwargs):
        """Main Notification System for Windows. Basically ported from go-toast/toast"""

        # Create the base
        self._top_ps1_script = f"""
[Windows.UI.Notifications.ToastNotificationManager, Windows.UI.Notifications, ContentType = WindowsRuntime] | Out-Null
[Windows.UI.Notifications.ToastNotification, Windows.UI.Notifications, ContentType = WindowsRuntime] | Out-Null
[Windows.Data.Xml.Dom.XmlDocument, Windows.Data.Xml.Dom.XmlDocument, ContentType = WindowsRuntime] | Out-Null
"""

    def _generate_notification_xml(
        self,
        application_id,
        notification_title,
        notification_subtitle,
        notification_icon,
        notification_audio,
    ):

        # Create the top <toast> element
        top_element = ElementTree.Element("toast")
        # set the duration for the top element
        top_element.set("duration", "short")

        # create the <visual> element
        visual_element = ElementTree.SubElement(top_element, "visual")

        # create <binding> element
        binding_element = ElementTree.SubElement(visual_element, "binding")
        # add the required attribute for this.
        # For some reason, go-toast set the template attribute to "ToastGeneric"
        # but it never worked for me.
        binding_element.set("template", "ToastImageAndText02")

        # create <image> element
        image_element = ElementTree.SubElement(binding_element, "image")
        # add an Id
        image_element.set("id", "1")
        # add the src
        image_element.set("src", notification_icon)

        # add the message and title

        title_element = ElementTree.SubElement(binding_element, "text")
        title_element.set("id", "1")
        title_element.text = notification_title

        message_element = ElementTree.SubElement(binding_element, "text")
        message_element.set("id", "2")
        message_element.text = notification_subtitle

        if notification_audio:
            # the user has provided his own audio file, no need to play the default sound.
            audio_element = ElementTree.SubElement(top_element, "audio")
            audio_element.set("silent", "true")

        # Great we have a generated XML notification.
        # We need to create the rest of the .ps1 file and dump it to the temporary directory

        generated_ps1_file = f"""
{self._top_ps1_script}
$APP_ID = "{application_id}"

$template = @"
{ElementTree.tostring(top_element, encoding="utf-8").decode('utf-8')}
"@

$xml = New-Object Windows.Data.Xml.Dom.XmlDocument
$xml.LoadXml($template)
$toast = New-Object Windows.UI.Notifications.ToastNotification $xml
[Windows.UI.Notifications.ToastNotificationManager]::CreateToastNotifier($APP_ID).Show($toast)
"""
        return generated_ps1_file

    def send_notification(
        self,
        notification_title,
        notification_subtitle,
        notification_icon,
        application_name,
        notification_audio,
        **kwargs,
    ):
        generated_file = self._generate_notification_xml(
            notification_title=notification_title,
            notification_subtitle=notification_subtitle,
            notification_icon=notification_icon,
            application_id=application_name,
            notification_audio=notification_audio,
        )

        if notification_audio:
            startupinfo = subprocess.STARTUPINFO()
            startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
            subprocess.Popen(
                [
                    "Powershell",
                    f'(New-Object Media.SoundPlayer "{notification_audio}").playsync()',
                ],
                startupinfo=startupinfo,
            )

        # open the temporary directory
        with tempfile.TemporaryDirectory() as temp_dir:
            generated_uuid_file = str(uuid.uuid4())
            with codecs.open(
                f"{temp_dir}/{generated_uuid_file}.ps1", "w", "utf_8_sig"
            ) as ps1_file:
                ps1_file.write(generated_file)
            # exceute the file
            startupinfo = subprocess.STARTUPINFO()
            startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
            subprocess.Popen(
                [
                    "Powershell",
                    "-ExecutionPolicy",
                    "Bypass",
                    "-File",
                    f"{generated_uuid_file}.ps1",
                ],
                cwd=temp_dir,
                startupinfo=startupinfo,
            ).wait()
        return True
